#____________________________________________________________________________
# Last Edit: 2019 12
#
# Script similar to the script used for analysis in [ARTICLE REFERENCE].
# This will fit the Model A described in the article to two groups 
# (Control and Intervention), for two variables (X and Y), with
# four equally-spaced observations across time. 

# Set the working directory of R using setwd() to be the same as all of the 
# supplemental R scripts and simulated data. 

rm(list=ls())
require(OpenMx)
source("201912 Additional Function.R")

#____________________________________________________________________________
# Load Data

	# These data consist of simulated covariance and mean matrices for two
	# groups. The data bear a resemblance to the
	# study data, but are not the data used in the analysis presented in the
	# article.
	
	# The data are assumed to consist of two variables (X and Y) measured
	# at four occasions (6, 12, 18, and 24 months), for each of two groups
	# (Control and Intervention)
	
	# Novel data can be analyzed by replacing the data in the mxData()
	# functions. Raw data are required for this script. Note that the column
	# names for the data need to be as follows: "MSTx6"  "MSTx12" "MSTx18" 
	# "MSTx24" "MSTy6"  "MSTy12" "MSTy18" "MSTy24". The use of "MST" 
	# differentiate the ManifeST variables from the state estimates of
	# X and Y. 
	
	load("SimulatedData.save")
		
	DataControl <- mxData(SimulatedData$Control,type="raw")
	DataIntervention <- mxData(SimulatedData$Intervention,type="raw")
		
	#Interval between observations (e.g., 6 months)
	ObservationInterval <- 6

	# The `LatentStepsSeries' defines a series of models with different
	# numbers of latent steps. These values should begin at 0 and increase
	# to a large number (e.g. ~100-150) until the model parameters appear
	# to be converging on asymptotic values. Models with 0-20 latent steps
	# will take on the order of seconds to minutes to run. Models with 
	# 50 and above latent steps can take on the order of 0.5 to many hours 
	# to run. The time require exponentially increases with the number of
	# latent steps. Running on a 2016 laptop with 8 processors, 100 latent 
	# steps required 3.2 hours.
	
	LatentStepsSeries <- c(0,1,3,5,10,20,50,100)
		
#____________________________________________________________________________
# Model A: Level X to Velocity Y

	# Use all available cores except for one
	mxOption(NULL, 'Number of Threads', parallel::detectCores()-1)
	
	for(steps in LatentStepsSeries) {
		
		# Creates components for an OpenMX model for the specified number
		# of latent steps. The program is based on RAM matrices. 
		source("201912 Level to Velocity Coupling.R")
		
		# If the number of steps is not equal to zero (the initial model), 
		# the estimated values from the last estimation will be substituted
		# into the new model as start values. 
		if(steps!=0) {
			Amatrix1 <- ParameterSub(Amatrix1,newEst$output$estimate)
			Smatrix1 <- ParameterSub(Smatrix1,newEst$output$estimate)
			Mmatrix1 <- ParameterSub(Mmatrix1,newEst$output$estimate)
			StochasticErr1 <- ParameterSub(StochasticErr1,
				newEst$output$estimate)
			ParamG1 <- ParameterSub(ParamG1,newEst$output$estimate)
			ParamG2 <- ParameterSub(ParamG2,newEst$output$estimate)
			}
	
		#Impled Covariance and Mean Algebras
		InvAlg <- mxAlgebra(solve(I-A),name="Inv")
		ECovAlg <- mxAlgebra(F%*%Inv%*%S%*%t(Inv)%*%t(F),name="ECov",
			dimnames=list(manifest,manifest))
		EMeanAlg <- mxAlgebra(t(F%*%Inv%*%t(M)),name="EMean",
			dimnames=list("",manifest))

		#Assemble components of the Control Group Model
		LatentModel1 <- mxModel("LatentModel1",DataControl,
			Amatrix1,Smatrix1,Fmatrix,Imatrix,Mmatrix1,StochasticErr1, 
			dtMat,NewAlg,InvAlg,ECovAlg,EMeanAlg,			
			mxExpectationNormal(covariance= "ECov", means = "EMean"),
			mxFitFunctionML()
			)
			
		#Assemble components of the Intervention Group Model
		LatentModel2 <- mxModel("LatentModel2",DataIntervention,
			Amatrix2,Smatrix2,Fmatrix,Imatrix,Mmatrix2,StochasticErr2, 
			dtMat,NewAlg,ParamG1,ParamG2,NewAlg2,InvAlg,ECovAlg,EMeanAlg,
			mxExpectationNormal(covariance= "ECov", means = "EMean"),
			mxFitFunctionML()
			)

		#Create Multigroup OpenMx Model
		mgFitFun <- mxFitFunctionMultigroup(c("LatentModel1","LatentModel2"))
		LatentModel <- mxModel(model="MultipleGroup",
			LatentModel1,LatentModel2,mgFitFun)
		
		#For step=0, ensure that each parameter has only a single start value.
		if(!exists("newEst")) { 
			LatentModel <- omxAssignFirstParameters(LatentModel) 
			}

		# Run OpenMx model, save the model within R to a unique object, save 
		# the model to a file with the number of steps in the file name, and 
		# provide some runtime information. 
		Start <- Sys.time()
		newEst <- mxRun(LatentModel,intervals=FALSE)
		ModelName <- paste("LevVelModel",steps,sep="")
		assign(ModelName,newEst)
		save(list=ModelName,file=paste(ModelName,".save",sep=""))
		cat("Steps:",steps,"\t Model runtime:",
			difftime(Sys.time(),Start,units="min"),"minutes \n\n")
		}





